"""
ExplosionPreventions app

App for explosionprevention analysis

"""
from analyses.analysisbase import models as base_models
from analyses.main import analysesmodules

from django.db import models
from django.db.models import signals as django_signals
from django.utils.functional import curry
from django.utils.translation import ugettext_lazy as _

from installations import models as installations_models

from utils.exchange import exchange
from utils.search.models import searches, Search
from utils.strings import add_suffix_to_string
from utils.trees import models as trees_models
from utils.versioning import models as versioning_models
from utils.versioning import versionedmptt

class AppConfig(base_models.AppConfig):
    namespace = "explosionprevention"
    tree_root_elementtype = "explosion"
    analysistype =  _('Avoiding ignition sources')
    analysistype_short =  _('Avoiding ignition sources__')
    scenario_displaypermission = 'explosionprevention.display_explosionpreventionscenarios'
    scenario_editpermission = 'explosionprevention.edit_explosionpreventionscenarios'
    suggestionlist_displaypermission = 'explosionprevention.display_explosionpreventionsuggestionlists'
    suggestionlist_editpermission = 'explosionprevention.edit_explosionpreventionsuggestionlists'
    scenarios_url = 'explosionprevention_scenarios'
    suggestionlists_url = 'explosionprevention_suggestionlists'
    addurl = 'explosionprevention_scenario_add'
    consequence_label = _('Consequence of ignition')

    @classmethod
    def get_models(cls):
        return {
            'suggestionlist': ExplosionPreventionSuggestionlist,
            'scenario': ExplosionPreventionScenario,
            'treeelement': ExplosionPreventionTreeElement,
            'event': ExplosionPreventionEvent,
            'protectionlayer': ExplosionPreventionProtectionLayer,
            'measure': ExplosionPreventionMeasure,
            'protectionlayer_measure': ExplosionPreventionPL_Measure,
            'attentionpoint': ExplosionPreventionAttentionPoint,
            'apsuggestionlist': ExplosionPreventionAPSuggestionlist,
       }

class ExplosionPreventionSuggestionlist(base_models.Suggestionlist):
    class Meta:
        permissions = (
            ("display_explosionpreventionsuggestionlists", "May display explosionprevention suggestion lists"),
            ("edit_explosionpreventionsuggestionlists", "May edit explosionprevention suggestion lists"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class ExplosionPreventionScenario(base_models.Scenario):
    suggestionlist = versioning_models.VersionedForeignKey(ExplosionPreventionSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="scenarios")

    class Meta(base_models.Scenario.Meta):
        ordering = ('type', 'name',)
        permissions = (
            ("display_explosionpreventionscenarios", "May display explosionprevention scenarios"),
            ("edit_explosionpreventionscenarios", "May edit explosionprevention scenarios"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

EVENT_ID = base_models.EVENT_ID
PROTECTIONLAYER_ID = base_models.PROTECTIONLAYER_ID

class ExplosionPreventionTreeElement(base_models.TreeElement):
    scenario = versioning_models.VersionedForeignKey(ExplosionPreventionScenario, verbose_name = _('Scenario'))

    appconfig = AppConfig


class ExplosionPreventionEvent(base_models.Event):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ExplosionPreventionMeasure(base_models.Measure):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ExplosionPreventionProtectionLayer(base_models.ProtectionLayer):
    measures = versioning_models.VersionedManyToManyField(ExplosionPreventionMeasure,
        verbose_name = _('Measures'),
        through="ExplosionPreventionPL_Measure",
        blank=True, null=True,
        related_name='protectionlayers')

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ExplosionPreventionPL_Measure(base_models.ProtectionLayer_Measure):
    protectionlayer = versioning_models.VersionedForeignKey(ExplosionPreventionProtectionLayer)
    measure = versioning_models.VersionedForeignKey(ExplosionPreventionMeasure)

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ExplosionPreventionAPSuggestionlist(base_models.APSuggestionlist):
    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class ExplosionPreventionAttentionPoint(base_models.AttentionPoint):
##    protectionlayer = versioning_models.VersionedForeignKey(ExplosionPreventionProtectionLayer,
##        verbose_name = _('Protection layer'), blank=True, null=True)
    measure = versioning_models.VersionedForeignKey(ExplosionPreventionMeasure, verbose_name = _('Measure'),
        blank=True, null=True,
        related_name="attentionpoints")
    suggestionlist = versioning_models.VersionedForeignKey(ExplosionPreventionAPSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="attentionpoints")

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission


make_version = curry(base_models.make_version, appconfig=AppConfig)
versioning_models.post_version.connect(make_version, sender=versioning_models.Version)

restore_version = curry(base_models.restore_version, appconfig=AppConfig)
versioning_models.restore_version.connect(restore_version, sender=versioning_models.Version)

analysesmodules.register(AppConfig, 50)

class ExplosionPrevention_SuggestionlistSearch(Search):
    permission = 'explosionprevention.display_explosionpreventionsuggestionlists'
    fields = ['name', 'description']
    model = ExplosionPreventionSuggestionlist
    name = _('Suggestion list')

searches.register(ExplosionPrevention_SuggestionlistSearch)

class ExplosionPrevention_ScenarioSearch(Search):
    permission = 'explosionprevention.display_explosionpreventionscenarios'
    fields = ['name', 'description', 'evaluation']
    model = ExplosionPreventionScenario
    name = _('Scenario')

searches.register(ExplosionPrevention_ScenarioSearch)

class ExplosionPrevention_EventSearch(Search):
    permission = 'explosionprevention.display_explosionpreventionscenarios'
    fields = ['name', 'description']
    model = ExplosionPreventionEvent
    name = _('Cause')

searches.register(ExplosionPrevention_EventSearch)

class ExplosionPrevention_ProtectionLayerSearch(Search):
    permission = 'explosionprevention.display_explosionpreventionscenarios'
    fields = ['name', 'description']
    model = ExplosionPreventionProtectionLayer
    name = _('Protection layer')

searches.register(ExplosionPrevention_ProtectionLayerSearch)

class ExplosionPrevention_MeasureSearch(Search):
    permission = 'explosionprevention.display_explosionpreventionscenarios'
    fields = ['name', 'tag', 'description']
    model = ExplosionPreventionMeasure
    name = _('Measure')

searches.register(ExplosionPrevention_MeasureSearch)

class ExplosionPrevention_APSuggestionlistSearch(Search):
    permission = 'explosionprevention.display_explosionpreventionsuggestionlists'
    fields = ['name', 'description']
    model = ExplosionPreventionAPSuggestionlist
    name = _('Attention point suggestion list')

searches.register(ExplosionPrevention_APSuggestionlistSearch)

class ExplosionPrevention_AttentionPointSearch(Search):
    permission = 'explosionprevention.display_explosionpreventionscenarios'
    fields = ['name', 'description']
    model = ExplosionPreventionAttentionPoint
    name = _('Attention point')

searches.register(ExplosionPrevention_AttentionPointSearch)
