"""
Comments app

! Depends on the versioning app

Objects for now need to be VersionedModel subclasses

"""
from django.db import models
from django.db.models import signals as django_signals
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from utils import reports as utils_reports
from utils.versioning import models as versioning_models

class CommentManager(versioning_models.VersionedModelManager):
    def comments_for_object(self, obj):
        object_type = ContentType.objects.get_for_model(obj)
        version = getattr(obj, '_version', obj.version) or versioning_models.WorkVersion
        return self.filter(content_type__pk=object_type.id,
                           object_id=obj.v_id).get_versionedobjects(version=version)

class Comment(versioning_models.VersionedModel):
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = versioning_models.VersionedGenericForeignKey('content_type', 'object_id')
    comment = models.TextField(verbose_name = _('Comment'))
    response = models.TextField(verbose_name = _('Response'), null=True, blank=True)
    user_response = models.ForeignKey(User, related_name="responded_comments", null=True, blank=True)

    objects = CommentManager()

    class Meta:
        verbose_name = _('Comment')
        verbose_name_plural = _('Comments')
        ordering = ['-date_created']
        permissions = (
            ("display_comments", "May display comments"),
            ("write_comments", "May write comments"),
            ("manage_comments", "May manage comments"),
        )

    def __unicode__(self):
        return u'%s %s %s' % (_('Comment by'), self.user_created.last_name, self.user_created.first_name)

    # urls

    @models.permalink
    def get_absolute_url(self):
        return ('comments_comment_display', (self.v_id,))

    @models.permalink
    def get_edit_url(self):
        return ('comments_comment_edit', (self.v_id,))

    @models.permalink
    def get_delete_url(self):
        return ('comments_comment_delete', (self.v_id,))

    @models.permalink
    def get_report_url(self):
        return ('comments_comment_report', (self.v_id,))

    def save(self, user, *args, **kwargs):
        """
        Custom save method
        Set response user
        """
        if not self.user_response and self.response:
            self.user_response = user
        super(Comment, self).save(user, *args, **kwargs)
    save.alters_data = True

    # permissions

    @classmethod
    def may_add(cls, user, parent=None):
        return user.has_perm('comments.write_comments')

    @classmethod
    def may_display(cls, user):
        return user.has_perm('comments.display_comments')

    def may_change(self, user):
        if self.version:
            return False
        if user.has_perm('comments.manage_comments'):
            return True
        if self.user_created == user:
            return True
        return False

    def may_delete(self, user):
        return self.may_change(user)

    @classmethod
    def get_listreport_formfields(cls, user):
        from django import forms
        res = [{'name': 'commentsheets',
            'field': forms.BooleanField(required=False, label=_('Comment sheets'),),
            'children': cls.get_report_formfields(user)}]
        return res

    @classmethod
    def get_listreport_css(cls, language=u'en', choices={}, **kwargs):
        return ['comments/comments_report.css']

    @classmethod
    def get_report_formfields(cls, user):
        return []

    def get_report_body(self, user, language=u'en', choices={}, level=1, subreport=False):
        from comments import forms as app_forms
        form = app_forms.CommentSheet(instance=self, omit_empty=True, auto_id=False)
        context = {'object': self, 'form': form, 'title': u'%s' % self, 'level': level}
        if subreport:
            html = utils_reports.make_pdf_body(context, language=language, template='comments/report.html')
        else:
            html = utils_reports.make_pdf_body(context, language=language)
        return [html,]

    @classmethod
    def get_report_css(cls, language=u'en', choices={}):
        return ["comments/report.css",]

    def get_report(self, user, language=u'en', choices={}):
        body_strings = self.get_report_body(user, language, choices)
        css_files = self.__class__.get_report_css(language, choices)
        return utils_reports.make_pdf(body_strings, css_files, language)

def make_comments_version(sender, instance, objects, **kwargs):
    if instance.published:
        filterkwarg={'published_dirty': True, 'version__isnull': True}
    else:
        filterkwarg={'dirty': True, 'version__isnull': True}
    if objects:
        for typelist in objects:
            if typelist:
                object_type = ContentType.objects.get_for_model(typelist[0])
                for item in Comment.objects.filter(**filterkwarg).filter(content_type__pk=object_type.id,
                           object_id__in=[item.v_id for item in typelist]):
                    item.make_version(instance)
    else:
        for item in Comment.objects.filter(**filterkwarg):
            item.make_version(instance)

versioning_models.post_version.connect(make_comments_version, sender=versioning_models.Version)

def restore_comments_version(sender, instance, objects, **kwargs):
    if objects:
        for typelist in objects:
            if typelist:
                object_type = ContentType.objects.get_for_model(typelist[0])
                for item in Comment.objects.filter(content_type__pk=object_type.id,
                           object_id__in=[item.v_id for item in typelist]):
                    item.restore_version(instance)
    else:
        for item in Comment.objects.get_newerobjects(instance):
            item.restore_version(instance)

versioning_models.restore_version.connect(restore_comments_version, sender=versioning_models.Version)


from utils.search.models import searches, Search
from utils.strings import trim_string

class Comments_CommentSearch(Search):
    permission = 'comments.display_comments'
    fields = ['comment',]
    model = Comment

    def get_label(self, object):
        return trim_string(object.comment,50)

    def get_url(self, object):
        from comments.views import display_url_for_obj
        return display_url_for_obj(object.content_object)


searches.register(Comments_CommentSearch)
