# comment views

from django.core.urlresolvers import reverse
from django.db.models.loading import get_model
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from comments import models as app_models
from comments import forms as app_forms
from comments import tables as app_tables

from utils import crud
from utils import forms as utils_forms
from utils import permissions
from utils.versioning import versionedcrud

class MixIn(object):
    "Common items for object views"
    model = app_models.Comment

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  [
        {'name': 'comments',
            'url': reverse('comments'),
            'label': _('Comments')},
        ]

class display(MixIn, versionedcrud.DisplayObjectView):
    form_class = app_forms.CommentSheet
    include_comments = False
    include_tasks = False

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

class edit(MixIn, versionedcrud.EditObjectView):
    def setup_from_args(self, *args, **kwargs):
        if self.request.user.has_perm('comments.manage_comments'):
            self.form_class = app_forms.CommentFullForm
        else:
            self.form_class = app_forms.CommentForm


class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = app_forms.CommentSheet

    def get_default_url(self, object):
        if object.content_object:
            return display_url_for_obj(object.content_object)
        return reverse('comments')

class report(MixIn, versionedcrud.ReportObjectView):
    pass

class add(versionedcrud.AddObjectView):
    """
    Add an comment to an object.
    """
    model = app_models.Comment
    form_class = app_forms.CommentForm
    parent_field = 'content_object'

    def __call__(self, request, app_label, model_name, *args, **kwargs):
        self.parent_class = get_model(app_label, model_name)
        return super(add, self).__call__(request, *args, **kwargs)

class display_for(versionedcrud.DisplayObjectView):
    allow_change = False
    allow_delete = False
    template = 'comments/display.html'
    allow_report = False
    def __call__(self, request, app_label, model_name, *args, **kwargs):
        self.model = get_model(app_label, model_name)
        return super(display_for, self).__call__(request, *args, **kwargs)

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        queryset = app_models.Comment.objects.comments_for_object(object)
        table = app_tables.CommentsTable(queryset, skipcolumns=['content_object',])
        buttons = []
        if app_models.Comment.may_add(self.request.user, object):
            buttons.append(
                {'url': add_url_for_obj(object),
                    'label': _('Add')},)
        table.buttons = buttons
        extra_context['table'] = table
        extra_context['breadcrumbs'] = [{'url': object.get_absolute_url(), 'label': u'%s' % object},]

def add_url_for_obj(obj):
    return reverse('comments_add', kwargs={
                        'app_label': obj._meta.app_label,
                        'model_name': obj._meta.model_name,
                        'object_id': obj.v_id
                    })

def display_url_for_obj(obj):
    return reverse('comments_display_for', kwargs={
                        'app_label': obj._meta.app_label,
                        'model_name': obj._meta.model_name,
                        'object_id': obj.v_id
                    })


class CommentFilter(crud.Filter):
    form = app_forms.CommentFilterForm
    def comment_response_filter(item):
        """
        callback function to supply parameters for filter form
        must return a list of qobjects and a dictionary of params
        """
        Qobjs = []
        params = {}
        if item == u'1':
        # no response
            Qobjs.append(Q(response__isnull=True) | Q(response=u''))
        elif item == u'2':
        # with response
            Qobjs.append(Q(response__isnull=False) & ~Q(response=u''))
        return Qobjs, params

    filters = [
                {'filteritem': 'comment', 'parameter': 'comment__icontains', 'type': 'normal'},
                {'filteritem': 'user', 'parameter': 'user_created__exact', 'type': 'normal'},
                {'filteritem': 'response', 'function': comment_response_filter, 'type': 'function'},
            ]

class CommentListObject(versionedcrud.TableListObject):
    defaultsort = '-date_created'
    table_class = app_tables.CommentsTable
    queryset = app_models.Comment.objects.all().select_related_versioned()
    title = _('Comments')
    table_classes = ['comments']

    def add_html(self, html_strings, queryset, user, language, choices):
        if choices.get('commentsheets'):
            html_strings.append('<pdf:nextpage>')
            for item in queryset:
                html_strings.extend(item.get_report_body(user, language, choices, level=2))
                html_strings.append('<hr />')

    def add_css(self, css_files, queryset, user, language, choices):
        css_files.extend(app_models.Comment.get_listreport_css(language, choices))
        if choices.get('commentsheets'):
            if queryset:
                css_files.extend(queryset[0].get_report_css(language, choices))

class comments(versionedcrud.TableListView):
    "List of comments"

    table_class = app_tables.CommentsTable
    csv_table_class = app_tables.CommentsCSVTable
    defaultsort = '-date_created'
    queryset = app_models.Comment.objects.all().select_related_versioned()
    paginate_by = 100
    title = _('Comments')
    filter = CommentFilter()
    allow_pdf = True
    allow_csv = True
    pdfselectform_class = utils_forms.BareListReportForm
    pdfsendmail = True
    listobject = CommentListObject()

    def get_pdfselectform(self):
        if self.request.method == 'POST':
            form = self.get_pdfselectform_class()(data=self.request.POST, user=self.request.user)
        else:
            form = self.get_pdfselectform_class()(user=self.request.user)
        return form

    def get_pdfselectform_class(self):
        return utils_forms.create_class_with_meta(self.pdfselectform_class, model=app_models.Comment)

    def add_to_extra_context(self, extra_context):
        super(comments, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  [
        {'name': 'comments',
            'url': reverse('comments'),
            'label': _('Comments')},
        ]

