"""
Users app

App to manage users and groups from django.contrib.auth

"""

from django.contrib.auth import models as auth_models
from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext_lazy as _

class UserManager(models.Manager):
    """
    Custom manager
    """
    def get_queryset(self):
        return super(UserManager, self).get_queryset().exclude(username__startswith='__')

class User(auth_models.User):
    objects = UserManager()
    class Meta:
        proxy = True
        verbose_name = _('User')
        verbose_name_plural = _('Users')
        permissions = (
            ("display_users", "May display users information"),
            ("edit_users", "May edit users information"),
        )

    # urls

    @models.permalink
    def get_absolute_url(self):
        return ('users_user', (self.id,))

    @models.permalink
    def get_edit_url(self):
        return ('users_user_edit', (self.id,))

    @models.permalink
    def get_delete_url(self):
        return ('users_user_delete', (self.id,))

    @models.permalink
    def get_change_password_url(self):
        return ('users_user_set_password', (self.id,))

    @models.permalink
    def get_temporary_password_url(self):
        return ('users_user_set_temporary_password', (self.id,))

    # permissions

    @classmethod
    def may_add(cls, user):
        return user.has_perm('auth.edit_users')

    def may_change(self, user):
        return user.has_perm('auth.edit_users')

    def may_delete(self, user):
        return False

class Group(auth_models.Group):
    class Meta:
        proxy = True
        verbose_name = _('Group')
        verbose_name_plural = _('Group')

    # urls

    @models.permalink
    def get_absolute_url(self, tab='group'):
        return ('users_group', (self.id, tab))

    @models.permalink
    def get_edit_url(self, tab='group'):
        return ('users_group_edit', (self.id,))

    @models.permalink
    def get_delete_url(self):
        return ('users_group_delete', (self.id,))

    # permissions

    @classmethod
    def may_add(cls, user):
        return user.has_perm('auth.edit_users')

    def may_change(self, user):
        return user.has_perm('auth.edit_users')

    def may_delete(self, user):
        if user.has_perm('auth.edit_users'):
            return self.user_set.count() == 0
        return False

def create_permissions(app, created_models, verbosity, **kwargs):
    from django.contrib.contenttypes.models import ContentType

    try:
        ct = ContentType.objects.get_by_natural_key('auth', 'user')
    except:
        ct = None
    if ct:
        try:
            auth_models.Permission.get(codename='display_users', content_type=ct)
        except:
            auth_models.Permission(codename='display_users', name='May display users information', content_type=ct)
        try:
            auth_models.Permission.get(codename='edit_users', content_type=ct)
        except:
            auth_models.Permission(codename='edit_users', name='May edit users information', content_type=ct)

signals.post_syncdb.connect(create_permissions)
