from django.contrib.auth.models import User, Group, Permission
from django.test import TestCase

import time

from utils import tests as utils_tests


class UsersTestCase(TestCase):
    fixtures = ['dummyusers', 'users']

    def setUp(self):
        from django.contrib.contenttypes.models import ContentType
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised to edit
        # 3 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_users','auth','user'),
            ('edit_users','auth','user'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_users','auth','user'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

class UsersUserTestCase(UsersTestCase):
    def test_user_views(self):

        base_url = '/users/users/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class users_list(utils_tests.TestDefinition):
            description = 'users list'
            url = base_url
            users = [1,2,]

            contains = [{'url': base_url + 'add/', 'users': [1,]},]

        class user(utils_tests.TestDefinition):
            description = 'display user'
            url = base_url + '5/'
            users = [1,2,]
            contains = [
                {'url': base_url + '5/edit/', 'users': [1,]},
                {'url': base_url + '5/delete/', 'users': []},
            ]

        class user_add(utils_tests.add_TestDefinition):
            description = 'add a user'
            url = base_url + 'add/'
            users = [1,]
            data = {
                "username": "bogus",
                "password1": "Xyzxyz1234",
                "password2": "Xyzxyz1234",
                "management-referer": base_url
            }
            def get_redirect_url(self):
                newid = User.objects.order_by('-id')[0].id
                return base_url + '%s/edit/' % newid

            def posttest(self, testcase, allowed):
                if allowed:
                    User.objects.order_by('-id')[0].delete()

        class user_edit(utils_tests.edit_TestDefinition):
            description = 'edit a user'
            url = base_url + '5/edit/'
            users = [1,]
            data = {
                "username": "A1",
                "management-referer": base_url + '5/'
            }
            redirect_url = base_url + '5/'

        class user_delete(utils_tests.delete_TestDefinition):
            description = 'delete a user'
            # Not allowed for any users!
            users = []
            redirect_url = base_url
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = User(username='to_delete')
                newitem.save()
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    User.objects.get(id=self.pk).delete()

        class user_setpassword(utils_tests.edit_TestDefinition):
            description = 'edit a users password'
            url = base_url + '5/edit/password/'
            users = [1,]
            data = {
                "new_password1": "Xyzxyz1234",
                "new_password2": "Xyzxyz1234",
                "management-referer": base_url + '5/'
            }
            redirect_url = base_url + '5/'
            def posttest(self, testcase, allowed):
                if allowed:
                    u = User.objects.get(pk=5)
                    u.set_password("test")
                    u.save()

        class user_changepassword(utils_tests.edit_TestDefinition):
            description = 'change password'
            url = '/users/change_password/'
            users = [1,2,3]
            data = {
                "old_password": "test",
                "new_password1": "Xyzxyz1234",
                "new_password2": "Xyzxyz1234",
                "management-referer": base_url + '5/'
            }
            redirect_url = '/'
            def posttest(self, testcase, allowed):
                if allowed:
                    u = User.objects.get(pk=testcase.user.id)
                    u.set_password("test")
                    u.save()

        tests = [
            users_list(),
            user(),
            user_add(),
            user_edit(),
            user_delete(),
            user_setpassword(),
            user_changepassword(),
        ]

        utils_tests.run_tests(self, users, tests)

class UsersGroupTestCase(UsersTestCase):
    def test_group_views(self):

        base_url = '/users/groups/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class groups(utils_tests.TestDefinition):
            description = 'groups list'
            url = base_url
            users = [1,2,]

            contains = [{'url': base_url + 'add/', 'users': [1,]},]

        class group_group(utils_tests.TestDefinition):
            description = 'display group tab group'
            url = base_url + '1/group/'
            users = [1,2,]
            contains = [
                {'url': base_url + '1/edit/', 'users': [1,]},
                {'url': base_url + '1/delete/', 'users': [1,]},
            ]

        class group_users(utils_tests.TestDefinition):
            description = 'display group tab users'
            url = base_url + '1/users/'
            users = [1,2,]
            contains = [
                {'url': base_url + '1/edit/', 'users': []},
                {'url': base_url + '1/delete/', 'users': []},
            ]

        class group_add(utils_tests.add_TestDefinition):
            description = 'add a group'
            url = base_url + 'add/'
            users = [1,]
            data = {
                "name": "bogus",
                "management-referer": base_url
            }
            redirect_url = base_url
            def posttest(self, testcase, allowed):
                if allowed:
                    Group.objects.order_by('-id')[0].delete()

        class group_edit(utils_tests.edit_TestDefinition):
            description = 'edit a group'
            url = base_url + '1/edit/'
            users = [1,]
            data = {
                "name": "A1",
                "management-referer": base_url + '1/group/'
            }
            redirect_url = base_url + '1/group/'

        class group_delete(utils_tests.delete_TestDefinition):
            description = 'delete a group'
            users = [1,]
            redirect_url = base_url
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = Group(name='to_delete')
                newitem.save()
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    Group.objects.get(id=self.pk).delete()

        tests = [
            groups(),
            group_group(),
            group_users(),
            group_add(),
            group_edit(),
            group_delete(),
        ]

        utils_tests.run_tests(self, users, tests)

