# user views

from django.contrib.auth import forms as auth_forms
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.utils.crypto import get_random_string

from django_auth_policy import forms as auth_policy_forms
from django_auth_policy.user_admin import StrictUserCreationForm
from django_auth_policy.models import PasswordChange

from users import forms as app_forms
from users import models as app_models
from users import tables as app_tables
from utils import crud
from utils.autocomplete import autocomplete

class MixIn(object):
    "Common items for object views"
    model = app_models.User

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  users_navigationlist()

class display(MixIn, crud.DisplayObjectView):
    form_class = app_forms.UserSheet
    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  users_navigationlist()
        if object.may_change(self.request.user):
            extra_context['defaultbuttons'].append(
                {'url': object.get_change_password_url(),
                'label': _('Change password')})
            extra_context['defaultbuttons'].append(
                {'url': object.get_temporary_password_url(),
                'label': _('Set a temporary password')})

class edit(MixIn, crud.EditObjectView):
    form_class = app_forms.UserForm
    def get_redirect(self, refform, object):
        return HttpResponseRedirect(object.get_absolute_url())

class set_password(MixIn, crud.EditObjectView):
    template = 'users/user_set_password.html'

    form_class = auth_policy_forms.StrictSetPasswordForm
    def get_form(self, object, **formargs):
        """
        Special because SetPasswordForm is not a modelform
        """
        if self.request.method == 'POST':
            form = self.form_class(object, data=self.request.POST, files=self.request.FILES, **formargs)
        else:
            form = self.form_class(object, initial=self.get_initial(object), **formargs)
        self.setup_form(form, object)
        self.form = form
        return form

    def save_form(self, form):
        object = form.save()
        return object

    def get_redirect(self, refform, object):
        return HttpResponseRedirect(object.get_absolute_url())

    def add_to_extra_context(self, extra_context, object):
        extra_context['title'] = _('Set password for user %s') % object.username
        rules = []
        for policy in self.form.password_strength_policy._policies:
            if policy.show_policy and policy.policy_text:
               rules.append({
                    'text': policy.policy_text,
                    'caption': policy.policy_caption,
                })
        extra_context['rules'] = rules

class change_password(set_password):
    """
    View to allow the user to change her password
    """
    form_class = auth_policy_forms.StrictPasswordChangeForm
    objectpermission = None
    def __call__(self, request, *args, **kwargs):
        """
        No need for object_id, we'll get the user object from the request
        """
        return super(change_password, self).__call__(request, *args, **kwargs)

    def lookup_object(self,*args, **kwargs):
        return self.request.user

    def get_redirect(self, refform, object):
        return HttpResponseRedirect('/')

    def add_to_extra_context(self, extra_context, object):
        super(change_password, self).add_to_extra_context(extra_context, object)
        if self.request.session.get('password_change_enforce', False):
            extra_context['enforced'] = self.request.session.get('password_change_enforce_msg', 'Password change required')

class set_temporary_password(MixIn, crud.EditObjectView):
    template = 'users/user_set_temporary_password.html'
    form_class = app_forms.TempPasswordForm

    def get_temporary_password(self):
        """Returns a random password."""
        from django.conf import settings
        # Characters used to generate temporary passwords
        allowed_chars = getattr(settings, 'TEMP_PASSWORD_CHARS',
                                'abcdefghijlkmnopqrstuvwxyz'
                                'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
                                '0123456789')
        # Temporary password length
        length = getattr(settings, 'TEMP_PASSWORD_LENGTH', 12)
        password = get_random_string(length, allowed_chars)
        return password

    def get_initial(self, object):
        return {'password1': self.get_temporary_password()}

    def save_form(self, form):
        password = form.cleaned_data['password1']
        PasswordChange.objects.create(user=self.object, is_temporary=True,
                                      successful=True)
        self.object.set_password(password)
        self.object.save()
        return self.object

    def get_redirect(self, refform, object):
        return HttpResponseRedirect(object.get_absolute_url())

    def add_to_extra_context(self, extra_context, object):
        extra_context['title'] = _('Set temporary password for user %s') % object.username

class delete(MixIn, crud.DeleteObjectView):
    form_class = app_forms.UserSheet

    def get_default_url(self, object):
        return reverse('users_users')

class add(MixIn, crud.AddObjectView):
    template = 'users/user_set_password.html'
    form_class = app_forms.StrictUserCreationForm
    def get_redirect(self, refform, object):
        return HttpResponseRedirect(object.get_edit_url())

class UserFilter(crud.Filter):
    form = app_forms.UserFilterForm
    filters = [
                {'filteritem': 'username', 'parameter': 'username__icontains', 'type': 'normal'},
            ]

class users(crud.TableListView):
    "List of users"

    table_class = app_tables.UsersTable
    defaultsort = 'username'
    queryset = app_models.User.objects.all().order_by('username')
    paginate_by = 100
    title = _('Users')
    filter = UserFilter()
    redirect_on_unique = True

    def add_to_extra_context(self, extra_context):
        super(users, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  users_navigationlist('users')
        if app_models.User.may_add(self.request.user):
            extra_context['tablebuttons'] = [{
                    'url': reverse('users_user_add'),
                    'label': _('Add'),
                }]

def users_navigationlist(current=None):
    navigationlist = [
        {'name': 'users',
            'url': reverse('users_users'),
            'label': _('Users')},
        {'name': 'groups',
            'url': reverse('users_groups'),
            'label': _('Groups')},
    ]
    nav = []
    for item in navigationlist:
        if current == item['name']:
            nav.append({'label': item['label']})
        else:
            nav.append({'label': item['label'], 'url': item['url']})
    return nav
