"""

Tests for the versioning app

"""
from django.test import TestCase
from django.contrib.auth.models import User
from utils.versioning.models import Version, WorkVersion
from utils import tests as utils_tests

class VersionViewsTestCase(TestCase):
    fixtures = ['dummyusers','versioning']

    def setUp(self):
        from django.contrib.auth.models import User, Permission
        from django.contrib.contenttypes.models import ContentType
        # give the users permissions

        app_label = 'versioning'

        # authorisations for users:
        # 1 = authorised
        # 2 = authorised to view all versions
        # 3 = authorised to view public versions
        # 4 = unauthorised

        u = User.objects.get(pk=1)
        u.user_permissions.add(Permission.objects.get(codename='display_published_versions', content_type=ContentType.objects.get(app_label=app_label, model='version')))
        u.user_permissions.add(Permission.objects.get(codename='display_versions', content_type=ContentType.objects.get(app_label=app_label, model='version')))
        u.user_permissions.add(Permission.objects.get(codename='edit_versions', content_type=ContentType.objects.get(app_label=app_label, model='version')))

        u = User.objects.get(pk=2)
        u.user_permissions.add(Permission.objects.get(codename='display_published_versions', content_type=ContentType.objects.get(app_label=app_label, model='version')))
        u.user_permissions.add(Permission.objects.get(codename='display_versions', content_type=ContentType.objects.get(app_label=app_label, model='version')))

        u = User.objects.get(pk=3)
        u.user_permissions.add(Permission.objects.get(codename='display_published_versions', content_type=ContentType.objects.get(app_label=app_label, model='version')))

    def test_version_views(self):

        base_url = '/versioning/versions/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3,4)

        class versions(utils_tests.tablelist_TestDefinition):
            description = 'list of versions'
            url = base_url
            users = [1,2,3]
            contains = [
                {'url': base_url + 'add/', 'users': [1,]},
            ]
            def extra_tests(self, testcase, response, allowed):
                # test rowcount here, should be different for user 3
                if testcase.user:
                    if testcase.user.id in [1,2]:
                        testcase.assertEquals(len(response.context['table'].rows), 2)
                    elif testcase.user.id in [3,]:
                        testcase.assertEquals(len(response.context['table'].rows), 1)

        class version_version_1(utils_tests.TestDefinition):
            # version 1 is not public
            description = 'display version 1 tab version'
            url = base_url + '1/version/'
            users = [1,2,]
            # no change or delete for nobody
            contains = [
                {'url': base_url + '1/edit/', 'users': []},
                {'url': base_url + '1/delete/', 'users': []},
                {'url': base_url + '1/items/', 'users': [1,2]},
            ]

        class version_version_2(utils_tests.TestDefinition):
            # version 2 is public
            description = 'display version 2 tab version'
            url = base_url + '2/version/'
            users = [1,2,3,]
            # no change or delete for nobody
            contains = [
                {'url': base_url + '2/edit/', 'users': []},
                {'url': base_url + '2/delete/', 'users': []},
                {'url': base_url + '2/items/', 'users': [1,2]},
            ]

        class version_items_1(utils_tests.TestDefinition):
            # version 1 is not public
            description = 'display version 1 tab items'
            url = base_url + '1/items/'
            users = [1,2,]
            # no change or delete for nobody
            contains = [
                {'url': base_url + '1/edit/', 'users': []},
                {'url': base_url + '1/delete/', 'users': []},
            ]

        class version_items_2(utils_tests.TestDefinition):
            # version 2 is public
            description = 'display version 2 tab items'
            url = base_url + '2/items/'
            users = [1,2,3]
            # no change or delete for nobody
            contains = [
                {'url': base_url + '2/edit/', 'users': []},
                {'url': base_url + '2/delete/', 'users': []},
            ]

        class version_add(utils_tests.add_TestDefinition):
            description = 'add a version'
            url = base_url + 'add/'
            users = [1,]
            data = {
                "name": "bogus",
                "management-referer": base_url
            }
            def get_redirect_url(self):
                l = u'%d' % (Version.objects.order_by('-id')[0].id)
                return base_url + l + '/create_version/'

            def extra_tests(self, testcase, response, allowed):
                if allowed:
                    newversion = Version.objects.order_by('-id')[0]
                    testcase.assertEquals(newversion.published, False)

            def posttest(self, testcase, allowed):
                if allowed:
                    Version.objects.order_by('-id')[0].delete()
                super(version_add, self).posttest(testcase, allowed)

        class version_publish(utils_tests.add_TestDefinition):
            description = 'publish a version'
            url = base_url + 'add/'
            users = [1,]
            data = {
                "name": "bogus",
                "published": True,
                "management-referer": base_url
            }
            def get_redirect_url(self):
                newversion = Version.objects.order_by('-id')[0]
                return base_url + '%s/create_version/' % newversion.id
            def extra_tests(self, testcase, response, allowed):
                if allowed:
                    newversion = Version.objects.order_by('-id')[0]
                    testcase.assertEquals(newversion.published, True)
            def posttest(self, testcase, allowed):
                if allowed:
                    Version.objects.order_by('-id')[0].delete()
                super(version_publish, self).posttest(testcase, allowed)

        class version_create_version_ajax(utils_tests.TestDefinition):
            description = 'ajax call to create the version objects'
            url = base_url + '1/create_version/ajax/'
            users = [1,]
            contains = [
                {'url': 'OK', 'users': [1,]},
            ]

        class version_activate(utils_tests.TestDefinition):
            description = 'ajax call to activate a version'
            url = base_url + '1/activate/'
            users = [1,2,]
            redirect_url = base_url + '1/version/'

            def runtests(self, testcase, allowed, contains, notcontains):
                """ Custom version because this view always redirects
                """
                response = testcase.client.get(self.url)
                if allowed:
                    testcase.assertRedirects(response, self.redirect_url)
                else:
                    testcase.assertRedirects(response, '/not_allowed/')
                return response

            def posttest(self, testcase, allowed):
                if allowed:
                    testcase.assertEqual(testcase.client.session['version_id'], 1)
                super(version_activate, self).posttest(testcase, allowed)

        class version_activate_public(utils_tests.TestDefinition):
            description = 'ajax call to activate a public version'
            url = base_url + '2/activate/'
            users = [1,2,3,]
            redirect_url = base_url + '2/version/'

            def runtests(self, testcase, allowed, contains, notcontains):
                """ Custom version because this view always redirects
                """
                response = testcase.client.get(self.url)
                if allowed:
                    testcase.assertRedirects(response, self.redirect_url)
                else:
                    testcase.assertRedirects(response, '/not_allowed/')
                return response

            def posttest(self, testcase, allowed):
                if allowed:
                    testcase.assertEqual(testcase.client.session['version_id'], 2)
                super(version_activate_public, self).posttest(testcase, allowed)

        class version_deactivate(utils_tests.TestDefinition):
            description = 'ajax call to deactivate all versions'
            url = base_url + 'deactivate/'
            users = [1,2,3]
            redirect_url = '/'

            def runtests(self, testcase, allowed, contains, notcontains):
                """ Custom version because this view always redirects
                """
                response = testcase.client.get(self.url)
                if allowed:
                    testcase.assertRedirects(response, self.redirect_url)
                else:
                    testcase.assertRedirects(response, '/not_allowed/')
                return response

            def posttest(self, testcase, allowed):
                if allowed:
                    testcase.assertEquals(testcase.client.session.get('version_id', 'dummy'), 'dummy')
                super(version_deactivate, self).posttest(testcase, allowed)

        class version_restore(utils_tests.add_TestDefinition):
            description = 'restore a version'
            url = base_url + '1/restore/'
            users = [1,]
            redirect_url = base_url + '1/version/'
            data = {}

        class changedobjects(utils_tests.tablelist_TestDefinition):
            description = 'list of changed objects'
            url = '/versioning/changedobjects/'
            users = [1,2,]

        class changedpublicobjects(utils_tests.tablelist_TestDefinition):
            description = 'list of changed public objects'
            url = '/versioning/changedpublicobjects/'
            users = [1,2,]

        tests = [
            versions(),
            version_version_1(),
            version_version_2(),
            version_items_1(),
            version_items_2(),
            version_add(),
            version_publish(),
            version_create_version_ajax(),
            version_activate(),
            version_activate_public(),
            version_deactivate(),
            version_restore(),
            changedobjects(),
            changedpublicobjects(),
        ]

        utils_tests.run_tests(self, users, tests)

