"""

Subclasses of the CRUD views, specifically for versioned model objects

"""
from django.contrib import messages
from django.http import HttpResponseRedirect, Http404
from django.shortcuts import get_object_or_404
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _

from utils import crud, permissions
from utils import forms as utils_forms
from utils.versioning import models as versioning_models
from utils.versioning import forms as versioning_forms
from utils.versioning.session import get_current_version

class DisplayObjectView(crud.DisplayObjectView):
    """
    Subclass of DisplayObjectView for versioned objects.
    """
    template = "versioning/display.html"
    include_tasks = True
    include_comments = True
    include_history = True
    include_attachments = False
    include_links = False

    def lookup_object(self,*args, **kwargs):
        (self.version, self.specialversion) = get_current_version(self.request)
        return get_versionedobject_or_404(self.model, self.version, v_id = self.object_id)

    def add_to_extra_context(self, extra_context, object):
        """
        Add versioning information to context
        """
        extra_context['version'] = self.version
        extra_context['specialversion'] = self.specialversion
        if self.include_history and self.request.user.has_perm('versioning.display_versions'):
            extra_context['include_history'] = True
        if self.include_tasks and self.request.user.has_perm('tasks.display_tasks'):
            extra_context['include_tasks'] = True
        if self.include_comments and self.request.user.has_perm('comments.display_comments'):
            extra_context['include_comments'] = True
        extra_context['include_attachments'] = self.include_attachments
        extra_context['include_links'] = self.include_links

class AddObjectView(crud.AddObjectView):
    """
    Subclass of AddObjectView for versioned objects.
    """
    def setup_parent_from_args(self, object_id=None, *args, **kwargs):
        (self.version, self.specialversion) = get_current_version(self.request)
        if self.parent_class and object_id:
            self.parent = get_versionedobject_or_404(self.parent_class, self.version, v_id = object_id)
            setattr(self.object, self.parent_field, self.parent)

    def save_instance(self, object):
        """
        Make sure the user is passed into the object save function
        """
        object.save(self.request.user)
        return object

class EditObjectView(crud.EditObjectView):
    """
    Subclass of EditObjectView for versioned objects.
    """
    def lookup_object(self,*args, **kwargs):
        (self.version, self.specialversion) = get_current_version(self.request)
        return get_versionedobject_or_404(self.model, self.version, v_id = self.object_id)

    def save_instance(self, object):
        """
        Make sure the user is passed into the object save function
        """
        object.save(self.request.user)
        return object

    def check_permission(self, object):
        # check for version mismatches
        if isinstance(self.object, versioning_models.VersionedModel):
            if self.version != versioning_models.WorkVersion or self.object.version:
                return permissions.redirect_to_not_allowed()
        return super(EditObjectView, self).check_permission(object)

class DeleteObjectView(crud.DeleteObjectView):
    """
    Subclass of DeleteObjectView for versioned objects.
    """
    def lookup_object(self,*args, **kwargs):
        (self.version, self.specialversion) = get_current_version(self.request)
        return get_versionedobject_or_404(self.model, self.version, v_id = self.object_id)

    def delete_instance(self, object):
        """
        Make sure the user is passed into the object delete function
        """
        object.delete(self.request.user)

    def check_permission(self, object):
        # check for version mismatches
        if isinstance(self.object, versioning_models.VersionedModel):
            if self.version != versioning_models.WorkVersion or self.object.version:
                return permissions.redirect_to_not_allowed()
        return super(DeleteObjectView, self).check_permission(object)

class CompareObjectView(crud.DisplayObjectView):
    """
    Generic view to compare two instances of a model.

    """
    viewtype = 'display'
    template = 'versioning/compare.html'
    objectpermission = None
    allow_change=None
    allow_delete=None

    def setup_tabs(self):
        # No tabs in compare views
        self.tabs = []

    def lookup_object(self,*args, **kwargs):
        return None

    def setup_from_args(self, *args, **kwargs):
        # make 0 into WorkVersion
        if self.version_id and int(self.version_id):
            self.version2 = get_object_or_404(versioning_models.Version, id = self.version_id)
        else:
            self.version2 = versioning_models.WorkVersion

    def __call__(self, request, object_id=None, version_id=None, *args, **kwargs):
        self.version_id = version_id
        return super(CompareObjectView, self).__call__(request, object_id, *args, **kwargs)

    def process_request(self):
        self.errormessages = []
        (self.version, self.specialversion) = get_current_version(self.request)
        self.object = get_versionedobject_or_404(self.model, self.version, v_id = self.object_id)
        self.object2 = get_versionedobject_or_404(self.model, self.version2, v_id = self.object_id)
        if self.form_class:
            self.form1 = self.get_form(self.object)
            self.form2 = self.get_form(self.object2)
        else:
            self.form1 = None
            self.form2 = None

    def add_to_extra_context(self, extra_context, object):
        extra_context['object'] = self.object
        extra_context['object2'] = self.object2
        extra_context['form1'] = self.form1
        extra_context['form2'] = self.form2
        extra_context['version2'] = self.version2
        extra_context['version'] = self.version
        extra_context['specialversion'] = self.specialversion
        buttons = [{
                    'url': object.get_absolute_url(),
                    'label': _('Back')}
            ]
        extra_context['buttons'] = buttons

class PublishObjectView(EditObjectView):
    form_class = versioning_forms.PublishVersionForm
    objectpermission = 'publish'

    def get_form(self, object, **formargs):
        return super(PublishObjectView, self).get_form(object, **formargs)

    def get_initial(self, object):
        return {'name': self.get_name(object)}

    def save_form(self, form):
        name = form.cleaned_data['name']
        instance = versioning_models.Version(name=name, published=True)
        instance.save(self.request.user, propagate=False)
        self.object.publish(self.request.user, instance)
        return self.object

    def set_message(self, object):
        message = self.message
        if not message:
            message = u'%s: %s' % (capfirst(_('published')), self.get_name(object))
        messages.info(self.request, message)

    def get_title(self, object):
        return u'%s: %s' % (capfirst(_('publish')), self.get_name(object))

    def get_name(self, object):
        return u'%s %s' % (self.model._meta.verbose_name, object)

class Filter(crud.Filter):
    """
    Subclass of Filter for versioned selects.
    """
    def __call__(self, request, queryset, **kwargs):
        (self.version, self.specialversion) = get_current_version(request)
        return super(Filter, self).__call__(request, queryset, **kwargs)

    def call_form(self, form, *args, **kwargs):
        return form(version=self.version, *args, **kwargs)

class ListView(crud.ListView):
    """
    Subclass of ListView for versioned objects.
    """
    def setup_from_args(self, *args, **kwargs):
        # get the active version
        (self.version, self.specialversion) = get_current_version(self.request)

    def add_to_extra_context(self, extra_context):
        extra_context['version'] = self.version
        extra_context['specialversion'] = self.specialversion

    def get_queryset(self, request):
        return self.queryset.get_versionedobjects(self.version)

    def set_task(self):
        from main.tasks import mail_pdf_for_list
        if not self.version or (self.version == versioning_models.WorkVersion):
            version_id = None
        else:
            version_id = self.version.pk
        try:
            mail_pdf_for_list.delay(self.listobject.__class__, version_id, self.request.user, self.filter.__class__, self.filterdata, self.language, self.choices, self.order_by)
        except:
            raise Http404

    def get_pdf(self):
        return self.listobject.get_report(self.version, self.request.user, self.filter, self.filterdata, self.language, self.choices, self.order_by)

class TableListView(crud.TableListView):
    """
    Subclass of TableListView for versioned objects.
    """
    def setup_from_args(self, *args, **kwargs):
        # get the active version
        (self.version, self.specialversion) = get_current_version(self.request)

    def add_to_extra_context(self, extra_context):
        extra_context['version'] = self.version
        extra_context['specialversion'] = self.specialversion

    def get_queryset(self, request):
        return self.queryset.get_versionedobjects(self.version)

    def set_task(self):
        from main.tasks import mail_pdf_for_list
        if not self.version or (self.version == versioning_models.WorkVersion):
            version_id = None
        else:
            version_id = self.version.pk
        try:
            mail_pdf_for_list.delay(self.listobject.__class__, version_id, self.request.user, self.filter.__class__, self.filterdata, self.language, self.choices, self.order_by)
        except:
            raise Http404

    def get_pdf(self):
        return self.listobject.get_report(self.version, self.request.user, self.filter, self.filterdata, self.language, self.choices, self.order_by)

class ListObject(crud.ListObject):
    def get_queryset(self):
        return self.queryset.get_versionedobjects(self.version)

    def get_report(self, version, user, filter, filterdata, language, choices, order_by):
        self.version = version
        return super(ListObject, self).get_report(user, filter, filterdata, language, choices, order_by)

class TableListObject(crud.TableListObject):
    def get_queryset(self):
        return self.queryset.get_versionedobjects(self.version)

    def get_report(self, version, user, filter, filterdata, language, choices, order_by):
        self.version = version
        return super(TableListObject, self).get_report(user, filter, filterdata, language, choices, order_by)

class NoObjectView(crud.NoObjectView):
    """
    Subclass of NoObjectView for versioned objects.
    """
    def setup_from_args(self, *args, **kwargs):
        # get the active version
        (self.version, self.specialversion) = get_current_version(self.request)

    def add_to_extra_context(self, extra_context):
        extra_context['version'] = self.version
        extra_context['specialversion'] = self.specialversion

class ReportObjectView(crud.ReportObjectView):
    """
    Subclass of ReportObjectView for versioned objects.
    """
    def lookup_object(self,*args, **kwargs):
        (self.version, self.specialversion) = get_current_version(self.request)
        return get_versionedobject_or_404(self.model, self.version, v_id = self.object_id)

    def get_form_class(self):
        return utils_forms.create_class_with_meta(self.form_class, model=self.model)

    def get_form(self, object, **formargs):
        return super(ReportObjectView, self).get_form(object, user=self.request.user)


class ReportNoObjectView(crud.ReportNoObjectView):
    """
    Subclass of ReportNoObjectView for versioned objects.
    """
    def setup_from_args(self, *args, **kwargs):
        # get the active version
        (self.version, self.specialversion) = get_current_version(self.request)

    def add_to_extra_context(self, extra_context):
        extra_context['version'] = self.version
        extra_context['specialversion'] = self.specialversion

class Inline(crud.Inline):
    """
    Subclass of Inline for versioned objects.
    """
    def save_instance(self, object):
        """
        Make sure the user is passed into the object save function
        """
        object.save(self.request.user)
        return object

    def save_formset(self, formset):
        return self.save_instances(formset.save(self.request.user, commit=False))


def get_versionedobject_or_404(model, version, *args, **kwargs):
    """
    Like django's get_object_or_404, but with versioning
    """
    try:
        return model.objects.get_versionedobject(version=version, *args, **kwargs)
    except model.DoesNotExist:
        raise Http404('No %s matches the given query.' % model._meta.object_name)
