"""
Test versioning app

"""
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import signals as django_signals
from django.utils.translation import ugettext_lazy as _
from utils.trees import models as trees_models
from utils.versioning import models as versioning_models



class Building(versioning_models.VersionedModel):
    name = models.CharField(max_length=100)

    def __unicode__(self):
        if self.version:
            return u'%s (%s)' % (self.name, self.version)
        return self.name

    def get_absolute_url(self):
        return '/building/%s/' % self.v_id

class CleaningLady(versioning_models.VersionedModel):
    name = models.CharField(max_length=100)
    def __unicode__(self):
        if self.version:
            return u'%s (%s)' % (self.name, self.version)
        return self.name

class Appartment(versioning_models.VersionedModel):
    name = models.CharField(max_length=100)
    building = versioning_models.VersionedForeignKey(Building)
    cleaningladies = versioning_models.VersionedManyToManyField("CleaningLady", through="Appartment_CleaningLady")
    inhabitants = versioning_models.VersionedManyToManyField(User, through="Appartment_User", target_field_name='inhabitant')

    def __unicode__(self):
        if self.version:
            return u'%s (%s)' % (self.name, self.version)
        return self.name

    def may_publish(self, user):
        """
        Only allow publishing when building published
        """
        if self.version:
            return False
        # check building
        if Building.objects.filter(v_id=self.building.v_id, published=True).count() == 0:
            return False
        return True

    def publish(self, user, instance):
        """
        Create a published version for an appartment
        """
        objects = []
        objects.append([self])
        filterkwarg={'published_dirty': True, 'version__isnull': True}

        # cleaningladies
        objects.append(list(Appartment_CleaningLady.objects.filter(**filterkwarg).filter(appartment=self)))
        objects.append(list(Appartment_User.objects.filter(**filterkwarg).filter(appartment=self)))

        instance.make_version_for_objects(objects)

class Appartment_CleaningLady(versioning_models.VersionedModel):
    appartment = versioning_models.VersionedForeignKey("Appartment")
    cleaninglady = versioning_models.VersionedForeignKey("CleaningLady")

    def __unicode__(self):
        n = u'%s - %s' % (self.appartment, self.cleaninglady)
        if self.version:
            n = u'%s (%s)' % (n, self.version)
        if self.deleted:
            n = u'%s (deleted)' % n
        return n

class Appartment_User(versioning_models.VersionedModel):
    appartment = versioning_models.VersionedForeignKey("Appartment")
    inhabitant = models.ForeignKey(User)

    def __unicode__(self):
        n = u'%s - %s' % (self.appartment, self.inhabitant)
        if self.version:
            n = u'%s (%s)' % (n, self.version)
        if self.deleted:
            n = u'%s (deleted)' % n
        return n

FOLDER_ID = 1
class DiskTreeElement(trees_models.TreeElement):
    TreeElementChoices = (
        (FOLDER_ID, _('Folder')),
    )

    def __unicode__(self):
        n = u'%s' % self.content_object
        if self.version:
            n = u'%s (%s)' % (n, self.version)
        if self.deleted:
            n = u'%s (deleted)' % n
        return n

    def get_root_nodes(self):
        # zoals secties, geen parent
        return self.__class__.tree.root_nodes()

class Folder(trees_models.TreeLeaf):
    def create_treeelement(self, **kwargs):
        return DiskTreeElement(elementtype=FOLDER_ID)

    def get_tree_class(self):
        return DiskTreeElement

def make_testv_version(sender, instance, objects, **kwargs):
    if instance.published:
        filterkwarg={'published_dirty': True, 'version__isnull': True}
    else:
        filterkwarg={'dirty': True, 'version__isnull': True}
    if not objects:
        # only version everything when objects are not specifically included
        for item in Building.objects.filter(**filterkwarg):
            item.make_version(instance)
        for item in Appartment.objects.filter(**filterkwarg):
            item.make_version(instance)
        for item in Folder.objects.filter(**filterkwarg):
            item.make_version(instance)
        for item in DiskTreeElement.tree.filter(**filterkwarg):
            item.make_version(instance)
        for item in CleaningLady.objects.filter(**filterkwarg):
            item.make_version(instance)
        for item in Appartment_CleaningLady.objects.filter(**filterkwarg):
            item.make_version(instance)
        for item in Appartment_User.objects.filter(**filterkwarg):
            item.make_version(instance)


versioning_models.post_version.connect(make_testv_version, sender=versioning_models.Version)

def restore_testv_version(sender, instance, objects, **kwargs):
    if not objects:
        for item in Building.objects.get_newerobjects(instance):
            item.restore_version(instance)
        for item in Appartment.objects.get_newerobjects(instance):
            item.restore_version(instance)
        for item in Folder.objects.get_newerobjects(instance):
            item.restore_version(instance)
        DiskTreeElement.tree.restore_version(instance)
        for item in CleaningLady.objects.get_newerobjects(instance):
            item.restore_version(instance)
        for item in Appartment_CleaningLady.objects.get_newerobjects(instance):
            item.restore_version(instance)
        for item in Appartment_User.objects.get_newerobjects(instance):
            item.restore_version(instance)

versioning_models.restore_version.connect(restore_testv_version, sender=versioning_models.Version)
