"""
custom widgets
"""

from django import forms
from django.conf import settings

import django.contrib.admin.widgets as adminwidgets

from django.forms.widgets import flatatt
from django.utils.encoding import smart_unicode
from django.utils.html import escape
from django.utils.safestring import mark_safe
import json
from django.utils import translation
from django.core.urlresolvers import reverse

# i18n
from django.utils.translation import ugettext_lazy as _

class DateWidget(forms.DateTimeInput):
    """
    like admin date widget with proper date formatting
    uses jquery datepicker
    requires DATE_OUTPUT_FORMAT setting in settings
    """
    class Media:
        css = { 'all': (
                settings.STATIC_URL + u'themes/redmond/jquery-ui-1.8.23.css',
            )}
        js = [
                settings.STATIC_URL + u'js/jquery-ui-1.8.23.min.js',
                settings.STATIC_URL + u'js/i18n/jquery.ui.datepicker-fr.js',
                settings.STATIC_URL + u'js/i18n/jquery.ui.datepicker-nl.js',
                settings.STATIC_URL + u'js/calendar.js',
            ]

    def __init__(self, attrs={}):
        super(DateWidget, self).__init__(format=settings.DATE_OUTPUT_FORMAT, attrs={'class': 'vDateField', 'size': '10'})

class FilteredSelectMultiple(adminwidgets.FilteredSelectMultiple):
    """
    proxy for admin widget
    """
    class Media:
        # order of css is important because adminbase overrides some styles from widgets
        css = { 'all': (
                settings.STATIC_URL + u'admin/css/widgets.css',
                settings.STATIC_URL + u'css/adminbase.css',
            )}
        js = [
                settings.STATIC_URL + u'admin/js/core.js',
                settings.STATIC_URL + u'admin/js/SelectBox.js',
                settings.STATIC_URL + u'admin/js/SelectFilter2.js',
            ]

class JQueryAutoCompleteTextInput(forms.TextInput):
    """
    like text input widget with jQuery autocomplete function
    based on djangosnippet 233, see www.djangosnippets.com
    """

    class Media:
        css = { 'all': (
                settings.STATIC_URL + u'themes/redmond/jquery-ui-1.8.23.css',
            )}
        js = [
                settings.STATIC_URL + u'js/jquery-ui-1.8.23.min.js',
            ]

    def __init__(self, source='', namedsource='', options={}, attrs={}):
        """source can be a list containing the autocomplete values or a
        string containing the url used for the XHR request.

        For available options see the autocomplete sample page::
        http://jquery.bassistance.de/autocomplete/"""

        self.options = 'minLength: 2'
        self.attrs = {'autocomplete': 'off'}
        self.source = source
        self.namedsource = namedsource

        if len(options) > 0:
            self.options = json.JSONEncoder().encode(options)

        self.attrs.update(attrs)

    def render_js(self, field_id):
        if self.namedsource:
            source = reverse(self.namedsource)
        else:
            source = self.source
        if isinstance(source, list):
            source = json.JSONEncoder().encode(source)
        elif isinstance(source, str):
            source = 'source: "%s"' % escape(source)
        else:
            raise ValueError('source type is not valid')

        options = ''
        if self.options:
            options += ',%s' % self.options

        return u'$(\'#%s\').autocomplete({%s%s});' % (field_id, source, options)

    def render(self, name, value=None, attrs=None):
        final_attrs = self.build_attrs(attrs, name=name)
        if value:
            final_attrs['value'] = escape(smart_unicode(value))

        if not self.attrs.has_key('id'):
            final_attrs['id'] = 'id_%s' % name

        return u'''<input type="text" %(attrs)s/>
        <script type="text/javascript"><!--//
        $(function(){%(js)s}) //-->
        </script>
        ''' % {
            'attrs' : flatatt(final_attrs),
            'js' : self.render_js(final_attrs['id']),
        }

class JQueryCombobox(forms.Select):
    """
    like select input widget with jQuery combobox filter function
    based on http://jqueryui.com/demos/autocomplete/#combobox
    """

    class Media:
        css = { 'all': (
                settings.STATIC_URL + u'themes/redmond/jquery-ui-1.8.23.css',
                settings.STATIC_URL + u'combobox/jquery.combobox.css',
            )}
        js = [
                settings.STATIC_URL + u'js/jquery-ui-1.8.23.min.js',
                settings.STATIC_URL + u'combobox/jquery.combobox.js',
            ]

    def render_js(self, field_id):
        return u'$(\'#%s\').combobox();' % (field_id, )

    def render(self, name, value, attrs=None, choices=()):
        if value is None: value = ''
        final_attrs = self.build_attrs(attrs, name=name)
        output = [u'<select%s>' % flatatt(final_attrs)]
        options = self.render_options(choices, [value])
        if options:
            output.append(options)
        output.append(u'</select>')
        output.append(u'''<script type="text/javascript"><!--//
         $(function(){%(js)s}) //-->
         </script>
         ''' % {
            'js' : self.render_js(final_attrs['id']),
        })
        return mark_safe(u'\n'.join(output))

class MarkItUpWidget(forms.Textarea):
    class Media:
        css = {
            'all' : (
                'markitup/skins/simple/style.css',
                'markitup/sets/markdown/style.css',
            )
        }
        js = (
            'markitup/jquery.markitup.js',
            'markitup/sets/markdown/set.js',
        )

    def render(self, name, value, attrs=None):
        html = super(MarkItUpWidget, self).render(name, value, attrs)
        html += u'''<script type="text/javascript">
                 $(document).ready(function() {
                   $("#%(id)s").markItUp(mySettings);
                 });
                 </script>''' % {'id': attrs['id']}
        return html

